#!/usr/bin/perl -w
#
# txt2pkt.pl - v0.0
# This is a script to create an outbound fidonet packet
# 
# Copyright (c) 2001-2002 Robert James Clay
# All Rights Reserved. This is free software;  you can redistribute
# it and/or modify it under the same terms as Perl itself.

# Setup

use strict;
use Getopt::Std;
use vars qw/ $opt_o $opt_f $opt_l $opt_x $opt_h /;

use FTN::Utility qw(&logging);

my ($Pktfile, $Debug, $Textfile, $Outdir, $serialno,
    $logfile, @lines);
 
my $progid = "wpkt";

getopts('o:f:l:xh');

if ($opt_h) {
    &help ();	# print usage & exit
    exit(1);
}

if ($opt_l) {
    $logfile=$opt_l;  # this needs to be the filename & path
    undef $opt_l;
} else {
   $logfile="txt2pkt.log";		# default log file
#    $logfile="STDOUT";		# default log file
}

if ($opt_x) {
    $Debug = $opt_x;	# set debug flag
    undef $opt_x;
    &logging($logfile, $progid, "Debug flag is set ...");
} else {
    $Debug = 0;
}
	
if ($opt_f) {
    $Textfile=$opt_f;  # this needs to be the filename & path
    undef $opt_f;
} else {
    $Textfile="testfile.txt";		# default test file
}
&logging($logfile, $progid, "The textfile is: $Textfile") if ($Debug);

if ($opt_o) {
    $Outdir=$opt_o;  # this needs to be the path
    undef $opt_o;
} else {
    $Outdir="out";	# default outbound dir
}


my $EOL = ($Debug) ? "\n" : "\r";      # for STDOUT

# This part is a definition of the PKT format per FTS-0001

# PKT Header; initialized variable are constants; comments are
#             in pack() notation

my $OrgNode = 544;                     # S
my $DestNode = 545;                    # S
my($Year,$Mon,$Day,$Hour,$Min,$Sec);    # SSSSSS
my $Baud = 0;                          # S
my $PktVer = 2;                        # S   Type 2 packet
my $OrgNet = 120;                      # S
my $DestNet = 120;                     # S
my $ProdCode = 0x100;                  # S   product code: ?
#my $PassWord = "test";                     # a8
my $PassWord = "";                     # a8
my $OrgZone = 1;                       # S
my $DestZone = 1;                      # S
my $AuxNet = $OrgNet;                  # S
my $CapWord = 0x100;                   # S   capability word: Type 2+
my $ProdCode2 = 0;                     # S   ?
my $CapWord2 = 1;                      # S   byte swapped cap. word
#my $OrgZone                           # S   (repeat)
#my $DestZone                          # S   (repeat)
my $OrgPoint = 0;                      # S
my $DestPoint = 0;                     # S
my $ProdSpec = 0;                      # L   ?

# MSG Header; duplicated variables are shown as comments to indicate
#             the MSG Header structure

#$PktVer                            # S   (repeat)
#$OrgNode = 544;                    # S   (repeat)
#$DestNode = 545;                   # S   (repeat)
#$OrgNet = 120;                     # S   (repeat)
#$DestNet = 120;                    # S   (repeat)
my $Attrib = 0;                        # S
my $Cost = 0;                          # S
#$DateTime = "";                    # a20 (this is a local())
my $To = "";                           # a? (36 max)
my $From = "";                         # a? (36 max)
my $Subj = "";                         # a? (72 max)

#"AREA: "                           # c6          }
my $Area = "";                         # a? (max?)   } all this is actually part
#possible kludges go here. 0x01<TAG>0x0D          } of the TEXT postions
#TEXT goes here. (ends with 2 0x0D's ???)         }

my $TearLine = "--- txt2pkt.pl v0.00\n";
my $Origin = " * Origin: ...txt2pkt.pl testing  ($OrgZone:$OrgNet/$OrgNode)$EOL";
my $SeenBy = "SEEN-BY: $OrgNet/$OrgNode$EOL";
my $Path = "\1PATH: $OrgNet/$OrgNode$EOL\0";          # note the \0 in $Path

# repeat MSG Headers/TEXT

# null (S) to mark done

# PKT name as per FTS

($Sec, $Min, $Hour, $Day, $Mon, $Year) = localtime();
$Year += 1900;

# create packet file name
my $PktFile = sprintf("%s/%02d%02d%02d%02d.PKT",$Outdir,$Day,$Hour,$Min,$Sec);

open(MSG,">$PktFile") || die;
binmode(MSG);

&pktheader();          # write PKT header

# create msg header 

$From="txt2pkt.pl v0.0";
$To="All";



open(FILE,$Textfile) || die" $!\n";

while(<FILE>)
 {
   
     #$_ =~ tr/\200-\377/\0-\177/;     # mask hi-bit characters
     #$_ =~ tr/\0-\037/\040-\100/;     # mask control characters
         
     next if (-s $Textfile) >32000;
 
         
     # read text body, translate LFs to CRs
         
     @lines = <FILE>;
     grep(s/\n/\r/,@lines);
         
     # kill leading blank lines
         
     shift(@lines) while ($lines[0] eq "\n");
         
     next if $#lines == -1;
         
         
     # write message to PKT file
         
     &msgheader();
         
     print MSG "$To\0";
     print MSG "$From\0";
     print MSG "$Subj\0";
     #print MSG "AREA:$Area$EOL";         # note: CR not nul
     print MSG "AREA:TESTECHO$EOL";         # note: CR not nul
         
     $serialno = unpack("%16C*",join('',@lines));
     $serialno = sprintf("%lx",$serialno + time);
     print MSG "\1MSGID: $OrgZone:$OrgNet/$OrgNode $serialno$EOL";
         
     print MSG @lines;
     print MSG $EOL,$TearLine,$Origin,$SeenBy,$Path;
     
     @lines = ();                        # all done with array (frees mem?)
     #print MSG "\0\0";                      # indicates no more messages

} 
   

print MSG "\0\0";                      # indicates no more messages

close(MSG);
close(FILE);


############################################
# write pktheader
# See FTS-0001
############################################
sub pktheader {
   my($buf);
   $buf = pack("SSSSSSSSSSSSSa8SSSSSSSSSSL",
       $OrgNode,$DestNode,
       $Year,$Mon,$Day,$Hour,$Min,$Sec,
       $Baud,$PktVer,
       $OrgNet,$DestNet,
       $ProdCode,$PassWord,
       $OrgZone,$DestZone,$AuxNet,
       $CapWord,$ProdCode2,$CapWord2,
       $OrgZone,$DestZone,
       $OrgPoint,$DestPoint,$ProdSpec);
   syswrite(MSG,$buf,58);
}

############################################
############################################
sub msgheader {
   my($buf);
   my($DateTime) = &datetime();
   $buf = pack("SSSSSSSa20",
	$PktVer,$OrgNode,$DestNode,$OrgNet,
        $DestNet,$Attrib,$Cost,$DateTime);
   print MSG $buf;
}


############################################
# form Fido date-time string 
############################################

sub datetime {
    my($sec, $min, $hour, $mday, $mon, $year);
    my @MoY = ('Jan','Feb','Mar','Apr','May','Jun',
            'Jul','Aug','Sep','Oct','Nov','Dec');
    ($sec, $min, $hour, $mday, $mon, $year) = localtime();
    sprintf("%02d %s %2d  %02d:%02d:%02d",
            $mday, $MoY[$mon], $year, $hour, $min, $sec);
}

############################################
# form a 'standard' extension
############################################
sub ext {
    my($sec, $min, $hour, $mday, $mon, $year, $wday);
    my @ExT = ('SU1','MO1','TU1','WE1','TH1','FR1','SA1');
    ($sec, $min, $hour, $mday, $mon, $year, $wday) = localtime();
    $ExT[$wday];
}

############################################
# Help
############################################
sub help {
    print "\nUsage:  txt2pkt.pl -o OutboundDir [-f textfile ] [-l logfilenamename] [-x] \n\n";
    print "   -f   text file to be writen to the packet.\n";
    print "   -o   Outbound Dir (defaults to 'out')\n";
    print "   -l   Log filename (defaults to STDOUT)\n";
    print "   -x	Debug Mode\n";
    print "\n";
}
